<?php

namespace App\domain\amadeus;

use App\Traits\FileStorageTrait;
use Exception;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Log;

class FareInformativePricingAction extends AmadeusRequestStateless
{
    use FileStorageTrait;

    protected Collection $passengers;
    protected array $segments;
    protected int $sequenceNumber;
    protected string $otherCompany;

    public function __construct()
    {
        parent::__construct();

        $this->sequenceNumber = 2;
    }

    public function execute(array $passenger, array $segments, string $otherCompany): array
    {

        try {
            $this->passengers = Collection::wrap($passenger);
            $this->segments = $segments;
            $this->otherCompany = $otherCompany;


            $response = $this->createCurlRequest();

            self::storeInFile("xml", "1.1-Informative-pricing-without-pnr-response", $response);

            $responseArray = $this->xmlResponseBodyToArray($response);

            self::storeInFile(
                "json", "1.1-Informative-pricing-without-pnr-response", json_encode($responseArray)
            );


            return $responseArray;

        } catch (Exception $exception) {
            $e = "Error in FareInformativePricingAction:" . $exception->getMessage();
            Log::error($e);
            echo $e;

            return ["error" => $e];
        }
    }

    protected function generateXMLRequest(): string
    {
        $xmlRequest = $this->xmlHeader() . '
            <soap:Body>
                <Fare_InformativePricingWithoutPNR>

                ' . $this->generatePassengerGroups() . '

                ' . $this->generateSegmentGroups() . '

                ' . $this->generatePricingGroup() . '

                </Fare_InformativePricingWithoutPNR>
            </soap:Body>

        </soap:Envelope>';

        self::storeInFile("xml", "1.1-Informative-pricing-without-pnr-request", $xmlRequest);

        return $xmlRequest;
    }

    protected function action(): string
    {
        return "http://webservices.amadeus.com/TIPNRQ_21_1_1A";
    }

    protected function generatePassengerGroups(): string
    {
        $adults = $this->passengers->where("passengerTypeCode", "ADT")->values();
        $children = $this->passengers->where("passengerTypeCode", "CHD")->values();
        $infants = $this->passengers->where("passengerTypeCode", "INF")->values();

        $quantity = 0;
        $measurementValue = 0;

        $xml = $this->getPassengerGroupString($adults, "ADT", $quantity, $measurementValue);
        if ($children->count() > 0) {
            $xml .= $this->getPassengerGroupString($children, "CHD", $quantity, $measurementValue);
        }
        if ($infants->count() > 0) {
            $xml .= $this->getPassengerGroupString($infants, "INF", $quantity, $measurementValue);
        }

        return $xml;
    }

    protected function getPassengerGroupString(
        Collection $passengers, string $type, int &$quantity, int &$measurementValue
    ): string
    {
        $quantity++;

        $xml = "
            <passengersGroup>
                <segmentRepetitionControl>
                    <segmentControlDetails>
                        <quantity>$quantity</quantity>
                        <numberOfUnits>{$passengers->count()}</numberOfUnits>
                    </segmentControlDetails>
                </segmentRepetitionControl>
                <travellersID>
        ";

        $passengerValue = 0;
        if ($type != "INF") {
            $passengerValue = $measurementValue;
        }
        foreach ($passengers as $index => $passenger) {
            $passengerValue = $passengerValue + 1;

            $xml .= "
                    <travellerDetails>
                        <measurementValue>$passengerValue</measurementValue>
                    </travellerDetails>
            ";
        }
        if ($type != "INF") {
            $measurementValue = $passengerValue;
        }

        $xml .= "
                </travellersID>
                <discountPtc>
                    <valueQualifier>$type</valueQualifier>
                    "

            . ($type === "INF" ? "<fareDetails><qualifier>766</qualifier></fareDetails>" : "") .

            "</discountPtc>
            </passengersGroup>
        ";

        return $xml;
    }

    protected function generateSegmentGroups(): string
    {
        $xml = "";

        $itemNumber = 0;
        foreach ($this->segments as $groupIndex => $group) {
            if (isset($group['groupOfFlights']['flightDetails']['flightInformation'])) {
                $flightInformation = $group['groupOfFlights']['flightDetails']['flightInformation'];

                $xml .= $this->generateSingleSegmentXML($flightInformation, $groupIndex, $itemNumber);

            } else {
                foreach ($group['groupOfFlights']['flightDetails'] as $index => $segment) {
                    $flightInformation = $segment['flightInformation'];

                    $xml .= $this->generateSingleSegmentXML($flightInformation, $groupIndex, $itemNumber);
                }
            }

//            foreach ($group['groupOfFlights']['flightDetails'] as $index => $segment) {
//                dump('$segment', $segment);
//                if (isset($segment['flightInformation'])) {
//                    $dateOfDeparture = $segment['flightInformation']['productDateTime']['dateOfDeparture'];
//                    $departureTime = $segment['flightInformation']['productDateTime']['timeOfDeparture'];
//                    $dateOfArrival = $segment['flightInformation']['productDateTime']['dateOfArrival'];
//                    $arrivalTime = $segment['flightInformation']['productDateTime']['timeOfArrival'];
//                    $bordPointLocationId = $segment['flightInformation']['location'][0]['locationId'];
//                    $offPointLocationId = $segment['flightInformation']['location'][1]['locationId'];
//                    $marketingCarrier = $segment['flightInformation']['companyId']['marketingCarrier'];
//                    $operatingCarrier = $segment['flightInformation']['companyId']['operatingCarrier'] ?? "";
//                    $flightNumber = $segment['flightInformation']['flightOrtrainNumber'];
//                    $bookingClass = $segment['flightInformation']['bookingClass'];
//                } else {
//                    $dateOfDeparture = $segment['productDateTime']['dateOfDeparture'];
//                    $departureTime = $segment['productDateTime']['timeOfDeparture'];
//                    $dateOfArrival = $segment['productDateTime']['dateOfArrival'];
//                    $arrivalTime = $segment['productDateTime']['timeOfArrival'];
//                    $bordPointLocationId = $segment['location'][0]['locationId'];
//                    $offPointLocationId = $segment['location'][1]['locationId'];
//                    $marketingCarrier = $segment['companyId']['marketingCarrier'];
//                    $operatingCarrier = $segment['companyId']['operatingCarrier'] ?? "";
//                    $flightNumber = $segment['flightOrtrainNumber'];
//                    $bookingClass = $segment['bookingClass'];
//                }
//            }

        }

        return $xml;
    }

    protected function generatePricingGroup(): string
    {
        return "
        <pricingOptionGroup>
            <pricingOptionKey>
                <pricingOptionKey>RP</pricingOptionKey>
            </pricingOptionKey>
        </pricingOptionGroup>

        <pricingOptionGroup>
            <pricingOptionKey>
                <pricingOptionKey>RU</pricingOptionKey>
            </pricingOptionKey>
        </pricingOptionGroup>

        <pricingOptionGroup>
            <pricingOptionKey>
                <pricingOptionKey>RLO</pricingOptionKey>
            </pricingOptionKey>
        </pricingOptionGroup>

        <pricingOptionGroup>
           <pricingOptionKey>
                <pricingOptionKey>FCO</pricingOptionKey>
           </pricingOptionKey>
           <currency>
               <firstCurrencyDetails>
                     <currencyQualifier>FCO</currencyQualifier>
                     <currencyIsoCode>GBP</currencyIsoCode>
               </firstCurrencyDetails>
           </currency>
        </pricingOptionGroup>

        <pricingOptionGroup>
            <pricingOptionKey>
                <pricingOptionKey>VC</pricingOptionKey>
            </pricingOptionKey>
            <carrierInformation>
                <companyIdentification>
                    <otherCompany>{$this->otherCompany}</otherCompany>
                </companyIdentification>
            </carrierInformation>
        </pricingOptionGroup>
        ";
    }

    protected function generateSingleSegmentXML(mixed $flightInformation, int|string $groupIndex, int &$itemNumber): string
    {
        $dateOfDeparture = $flightInformation['productDateTime']['dateOfDeparture'];
        $departureTime = $flightInformation['productDateTime']['timeOfDeparture'];
        $dateOfArrival = $flightInformation['productDateTime']['dateOfArrival'];
        $arrivalTime = $flightInformation['productDateTime']['timeOfArrival'];
        $bordPointLocationId = $flightInformation['location'][0]['locationId'];
        $offPointLocationId = $flightInformation['location'][1]['locationId'];
        $marketingCarrier = $flightInformation['companyId']['marketingCarrier'];
        $operatingCarrier = $flightInformation['companyId']['operatingCarrier'] ?? "";
        $flightNumber = $flightInformation['flightOrtrainNumber'];
        $bookingClass = $flightInformation['bookingClass'];

        $itemNumber++;

        return "
                <segmentGroup>
                    <segmentInformation>
                        <flightDate>
                            <departureDate>$dateOfDeparture</departureDate>
                            <departureTime>$departureTime</departureTime>
                            <arrivalDate>$dateOfArrival</arrivalDate>
                            <arrivalTime>$arrivalTime</arrivalTime>
                        </flightDate>
                        <boardPointDetails>
                          <trueLocationId>$bordPointLocationId</trueLocationId>
                        </boardPointDetails>
                        <offpointDetails>
                          <trueLocationId>$offPointLocationId</trueLocationId>
                        </offpointDetails>
                        <companyDetails>
                          <marketingCompany>$marketingCarrier</marketingCompany>
                          <operatingCompany>$operatingCarrier</operatingCompany>
                        </companyDetails>
                        <flightIdentification>
                          <flightNumber>$flightNumber</flightNumber>
                          <bookingClass>$bookingClass</bookingClass>
                        </flightIdentification>

                       <flightTypeDetails>
                          <flightIndicator>" . ($groupIndex + 1) . "</flightIndicator>
                       </flightTypeDetails>
                       <itemNumber>$itemNumber</itemNumber>
                    </segmentInformation>
                </segmentGroup>
            ";
    }
}
